/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/


#ifndef EXTENSIONS_REFINE_OPERATION_H
#define EXTENSIONS_REFINE_OPERATION_H

#include <vector>
#include "AMDiS_fwd.h"

namespace AMDiS { namespace extensions {

  /// Operations performed before and after refinement/ coarsening
  struct StandardRefineOperation
  {
    virtual ~StandardRefineOperation() {};
    virtual void beforeRefine(AdaptInfo* adaptInfo, Flag markFlag) {}
    virtual void beforeCoarsen(AdaptInfo* adaptInfo, Flag markFlag) {}
    virtual void afterCoarsen(AdaptInfo* adaptInfo, Flag markFlag) {}  
  };

  /// vector of operations performed before and after refinement/ coarsening
  struct CoupledRefineOperation : public StandardRefineOperation
  {
    CoupledRefineOperation(std::vector<StandardRefineOperation*> operations_) 
      : operations(operations_) {}
      
    void beforeRefine(AdaptInfo* adaptInfo, Flag markFlag) override 
    {
      for (size_t i = 0; i < operations.size(); i++)
	  operations[i]->beforeRefine(adaptInfo, markFlag);
    }
    
    void beforeCoarsen(AdaptInfo* adaptInfo, Flag markFlag) override
    {
      for (size_t i = 0; i < operations.size(); i++)
	operations[i]->beforeCoarsen(adaptInfo, markFlag);
    }
    void afterCoarsen(AdaptInfo* adaptInfo, Flag markFlag) override
    {
      for (size_t i = 0; i < operations.size(); i++)
	operations[i]->afterCoarsen(adaptInfo, markFlag);
    }

  protected:
    std::vector<StandardRefineOperation*> operations;
  }; 

} }

#endif
