#include "SingularDirichletBC2.h"

namespace AMDiS { namespace extensions {
  
  namespace details {

// BoundaryType
template<>
void getIndicesAux<BoundaryTypeContainer>(
	BoundaryTypeContainer& pos, 
	BoundaryType boundary_nr,
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indices,
	boost::false_type is_not_abstract_fct)
{
  details::getBoundaryIndices(feSpace, pos.b, NULL, indices);
}

// double
template<>
void getIndicesAux<DofIndex>(
	DofIndex& pos, 
	BoundaryType boundary_nr,
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indices,
	boost::false_type is_not_abstract_fct)
{
  indices[pos.idx] = 0.0;
}

// WorldVector
template<>
void getIndicesAux<WorldVector<double> >(
	WorldVector<double>& pos, 
	BoundaryType boundary_nr,
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indices,
	boost::false_type is_not_abstract_fct)
{
  DegreeOfFreedom idx_ = -1;
  DOFVector<double> tmp(feSpace, "tmp");
  if (tmp.getDofIdxAtPoint(pos, idx_) && idx_ >= 0)
    indices[idx_] = 0.0;
}
  
// AbstractFunction (signedDistFct)
template<>
void getIndicesAux<AbstractFunction<double, WorldVector<double> > >(
	AbstractFunction<double, WorldVector<double> >& pos, 
	BoundaryType boundary_nr,
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indices,
	boost::true_type is_abstract_fct)
{
  details::getImplicitIndices(feSpace, &pos, indices);
}

// DOFVector (signedDistDOF)
template<>
void getIndicesAux<DOFVector<double> >(
	DOFVector<double>& pos, 
	BoundaryType boundary_nr,
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indices,
	boost::false_type is_not_abstract_fct)
{
  details::getImplicitIndices(feSpace, &pos, indices);
}

// Meshindicator
template<>
void getIndicesAux<AbstractFunction<bool, WorldVector<double> > >(
	AbstractFunction<bool, WorldVector<double> >& pos, 
	BoundaryType boundary_nr,
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indices,
	boost::true_type is_abstract_fct)
{
  details::getBoundaryIndices(feSpace, boundary_nr, &pos, indices);
}

// -----------------------------------------------------------------------------

// double
template<>
void getValuesAux<double>(
	const double &val, 
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indicesValues,
	boost::false_type is_not_abstract_fct,
	int depth)
{
  std::map<DegreeOfFreedom, double>::iterator it;
  for (it = indicesValues.begin(); it != indicesValues.end(); it++)
    it->second = val;
}

template<>
void getValuesAux<AbstractFunction<double, WorldVector<double> > >(
	const AbstractFunction<double, WorldVector<double> >& val, 
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indicesValues,
	boost::true_type is_abstract_fct,
	int depth)
{
  DOFVector<WorldVector<double> > x(feSpace, "coords");
  feSpace->getMesh()->getDofIndexCoords(x);
  std::map<DegreeOfFreedom, double>::iterator it;
  for (it = indicesValues.begin(); it != indicesValues.end(); it++)
    it->second = val(x[it->first]);
}

// DOFVector<double>
template<>
void getValuesAux<DOFVector<double> >(
	const DOFVector<double>& val, 
	const FiniteElemSpace *feSpace, 
	std::map<DegreeOfFreedom, double> &indicesValues,
	boost::false_type is_not_abstract_fct,
	int depth)
{
  std::map<DegreeOfFreedom, double>::iterator it;
  for (it = indicesValues.begin(); it != indicesValues.end(); it++)
    it->second = val[it->first];
}

}

} } // end namespaces
