/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef CAHN_HILLIARD_H
#define CAHN_HILLIARD_H

#include "AMDiS.h"
#include "BaseProblem_RB.h"
#include "chns.h"

namespace AMDiS { namespace base_problems {

class CahnHilliardNavierStokes_TwoPhase_RB : public BaseProblem_RB
{
public: // definition of types

  typedef BaseProblem_RB super;

public: // public methods

  CahnHilliardNavierStokes_TwoPhase_RB(const std::string &name_);
  ~CahnHilliardNavierStokes_TwoPhase_RB();

  virtual void initData();

  virtual void transferInitialSolution(AdaptInfo *adaptInfo);  
  virtual void closeTimestep(AdaptInfo *adaptInfo);
  
  double getEpsilon() { return eps; }
  int getDoubleWellType() { return doubleWell; }

  DOFVector<WorldVector<double> >* getVelocity()
  {
    return velocity;
  }
  
  virtual void fillOperators();
  virtual void fillBoundaryConditions() {}
//   virtual void addNonlinearTerm(int i);
  virtual void addStressTerm(int i);
  
  
  
//   virtual void addMaterialDerivativeConst(int i, int j,double factor = 1.0); 	// rho = const
//   virtual void addMaterialDerivativeNonConst(int i, int j);			// rho = rho(x)
//   virtual void addLaplaceTermConst(int i);					// viscosity = const
//   virtual void addLaplaceTermNonConst(int i);					// viscosity = viscosity(x)

protected: // protected variables

  DOFVector<WorldVector<double> >* velocity;

  void calcVelocity()
  {
    if (dow == 1)
      transformDOF(prob->getSolution()->getDOFVector(2), velocity, new AMDiS::Vec1WorldVec<double>);
    else if (dow == 2)
      transformDOF(prob->getSolution()->getDOFVector(2), prob->getSolution()->getDOFVector(3), velocity, new AMDiS::Vec2WorldVec<double>);
    else if (dow == 3)
      transformDOF(prob->getSolution()->getDOFVector(2), prob->getSolution()->getDOFVector(3), prob->getSolution()->getDOFVector(4), velocity, new AMDiS::Vec3WorldVec<double>);
  }
  
  bool useMobility;
  bool useConservationForm;

  unsigned dim;
  
  int laplaceType;
  int nonLinTerm;
  int doubleWell;

  // navierStokes parameters
  double oldTimestep;
  double viscosity1;
  double viscosity2;
  double density1;
  double density2;
  double c;
  
  // Cahn-Hilliard parameters
  double gamma;
  double eps;
  double minusEps;
  double epsInv;
  double minusEpsInv;
  double epsSqr;
  double minusEpsSqr;
  
  double sigma;		// coupling parameter to calculate the surface tension
  double surfaceTension;// := sigma/epsilon
  double surfaceTensionFactor;// := epsilon*2*sqrt(2)/(3*sigma)
  
  WorldVector<double> force;

  FileVectorWriter *fileWriter;
};

} }

#endif // CAHN_HILLIARD_H
