/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef CAHN_HILLIARD_RB_H
#define CAHN_HILLIARD_RB_H

#include "AMDiS.h"
#include "BaseProblem_RB.h"
#include "ExtendedProblemStat.h"
#include "HL_SignedDistTraverse.h"
#include "chns.h"

namespace AMDiS { namespace base_problems {

class CahnHilliard_RB : public BaseProblem_RB
{
public: // definition of types

  typedef BaseProblem_RB super;

public: // public methods

  CahnHilliard_RB(const std::string &name_);
  ~CahnHilliard_RB();

  virtual void initData();
  virtual void solveInitialProblem(AdaptInfo *adaptInfo);

  double getEpsilon() { return eps; }
  int getDoubleWellType() { return doubleWell; }

protected: // protected methods

  virtual void fillOperators();
  virtual void fillBoundaryConditions() {}

protected: // protected variables

  HL_SignedDistTraverse *reinit;

  bool useMobility;

  unsigned dim;

  int doubleWell;

  double gamma;
  double eps;
  double minusEps;
  double epsInv;
  double minusEpsInv;
  double epsSqr;
  double minusEpsSqr;
};

} }

#endif // CAHN_HILLIARD_RB_H
