namespace AMDiS { namespace base_problems {
  
namespace detail
{

  template<typename P>
  PhaseFieldCrystal_Phase<P>::PhaseFieldCrystal_Phase(const std::string &name_) 
    : super(name_),
      phase(NULL),
      phaseDisturbed(NULL)
  { }


  template<typename P>
  PhaseFieldCrystal_Phase<P>::~PhaseFieldCrystal_Phase()
  {
    if (phase) {
      delete phase;
      phase = NULL;
    }
    if (phaseDisturbed) {
      delete phaseDisturbed;
      phaseDisturbed = NULL;
    }
  }


  template<typename P>
  void PhaseFieldCrystal_Phase<P>::fillOperators()
  { 
    const FiniteElemSpace* feSpace0 = self::prob->getFeSpace(0);
    const FiniteElemSpace* feSpace1 = self::prob->getFeSpace(1);
    const FiniteElemSpace* feSpace2 = self::prob->getFeSpace(2);

    DOFVector<double>* rho = super::getStepSolution()->getDOFVector(1);
    DOFVector<double>* rho_old = super::getOldSolution()->getDOFVector(1);

    // dt(rho) = laplace(mu) - u*grad(rho)
    // -----------------------------------
    Operator *opM11 = new Operator(feSpace1, feSpace1);
    addZOT(opM11, valueOf(phaseDisturbed));
    self::prob->addMatrixOperator(opM11, 1, 1);

    Operator *opLM = new Operator(feSpace1, feSpace0);
    if (super::mobility_type == 1)
      addSOT(opLM, valueOf(phaseDisturbed) * max(abs_(valueOf(rho) + 1.5)*super::M0, 1.e-5));
    else if (super::mobility_type == 2)
      addSOT(opLM, valueOf(phaseDisturbed) * max(valueOf(rho)*super::M0, 1.e-5));
    else
      addSOT(opLM, valueOf(phaseDisturbed) * super::M0);
    self::prob->addMatrixOperator(opLM, 1, 0, self::getTau(), self::getTau()); // -laplace(mu)

    // . . . vectorOperators . . . . . . . . . . . . . . .
    Operator *opMold = new Operator(feSpace1);
    addZOT(opMold, valueOf(rho_old) * valueOf(phaseDisturbed));
    self::prob->addVectorOperator(opMold, 1);

    // mu-2*nu-laplace(nu)-(1+r)*rho = (rho_old^3) + ExtPot - eps^2/(rho_old+0.9)
    // ----------------------------------------------------------------------
    Operator *op01 = new Operator(feSpace0, feSpace1);
    addZOT(op01, valueOf(phaseDisturbed) * ( -(1.0 + ref_(super::r)) - 3.0 * pow<2>(valueOf(rho)) ) );
    addSOT(op01, 2.0 * valueOf(phaseDisturbed));
    self::prob->addMatrixOperator(op01, 0, 1);

    Operator *op00 = new Operator(feSpace0, feSpace0);
    addZOT(op00, valueOf(phaseDisturbed));
    self::prob->addMatrixOperator(op00, 0, 0);

    Operator *op02 = new Operator(feSpace0, feSpace2);
    addSOT(op02, valueOf(phaseDisturbed));
    self::prob->addMatrixOperator(op02, 0, 2);

    // . . . vectorOperators . . . . . . . . . . . . . . .
    Operator *op0 = new Operator(feSpace0);
    addZOT(op0, -2.0 * valueOf(phaseDisturbed) * pow<3>(valueOf(rho)) );
    self::prob->addVectorOperator(op0, 0);

    // 0 = nu-laplace(rho)
    // -------------------
    Operator *op21 = new Operator(feSpace2, feSpace1);
    addSOT(op21, valueOf(phaseDisturbed));
    self::prob->addMatrixOperator(op21, 2, 1); // -laplace(rho)

    Operator *op22 = new Operator(feSpace2, feSpace2);
    addZOT(op22, valueOf(phaseDisturbed));
    self::prob->addMatrixOperator(op22, 2, 2); // nu
  }


  template<typename P>
  double PhaseFieldCrystal_Phase<P>::calcEnergy()
  {
    DOFVector<double>* rho = self::prob->getSolution(1);
    DOFVector<double>* mu = self::prob->getSolution(0);
    
    return integrate( valueOf(phase) * ( 0.5*valueOf(rho)*valueOf(mu) - 0.25*pow<4>(valueOf(rho)) ) );
  }

} // end namespace detail

} }
