/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef CAHN_HILLIARD_H_
#define CAHN_HILLIARD_H_

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"
#include "chns.h"

using namespace AMDiS;

class CahnHilliard_ : public BaseProblem<ProblemStat>
{
public: // definition of types

  typedef BaseProblem<ProblemStat> super;

public: // public methods

  CahnHilliard_(const std::string &name_, bool createProblem = true);
  ~CahnHilliard_();

  virtual void initData() {};
  virtual void solveInitialProblem(AdaptInfo *adaptInfo);

  double getEpsilon() { return eps; }
  int getDoubleWellType() { return doubleWell; }

  virtual void fillOperators();
  virtual void fillBoundaryConditions() {}

protected: // protected variables

  bool useMobility;

  unsigned dim;

  int doubleWell;

  double gamma;
  double eps;
  double minusEps;
  double epsInv;
  double minusEpsInv;
  double epsSqr;
  double minusEpsSqr;
};

#endif // CAHN_HILLIARD_H_
