#include "AMDiS.h"

using namespace AMDiS;

class Circle : public AbstractFunction<double, WorldVector<double> > 
{
public:
  Circle(double radius_, WorldVector<double> midPoint_) 
    : radius(radius_), 
      midPoint(midPoint_) 
  {}

  Circle(double radius_) 
    : radius(radius_) 
  { 
    midPoint.set(0.0); 
  }

  double operator()(const WorldVector<double>& x) const
  {
    double result = 0.0;   
    for (int k = 0; k < x.getSize(); k++)
      result += sqr(x[k] - midPoint[k]);
    
    result = sqrt(result) - radius;
    return result;
  };
private:
  double radius;
  WorldVector<double> midPoint;
};


class InitialPhaseField : public AbstractFunction<double, WorldVector<double> >
{
public:
  InitialPhaseField(AbstractFunction<double,WorldVector<double> > *dist_) 
    : dist(dist_) 
  {}

  double operator()(const WorldVector<double> &x) const
  {
    return -tanh((*dist)(x) / sqrt(2.0));
  }
  
private:
  AbstractFunction<double,WorldVector<double> > *dist;
};


class MyProblemInstat : public ProblemInstat
{
public:
  MyProblemInstat(std::string name, ProblemStat &probStat)
    : ProblemInstat(name, probStat),
      hInitInterface(0.0),
      hInterface(0.0),
      refInterface(0),
      rhoInitial(0.0),
      serial(0)
  {
    FUNCNAME("MyProblemInstat::MyProblemInstat()");

    Parameters::get("dendrites->h init interface", hInitInterface);  
    Parameters::get("dendrites->h interface", hInterface);  
    Parameters::get("dendrites->ref interface", refInterface);
    Parameters::get("dendrites->karma model->rho initial", rhoInitial);
    Parameters::get("dendrites->space->input->read serialization", serial);

    TEST_EXIT(rhoInitial > 0.0)("Initial rho must be greater than 0!\n");
  }

  void solveInitialProblem(AdaptInfo *adaptInfo);

  void closeTimestep(AdaptInfo *adaptInfo);

private:
  inline int sign(double value)
  {
    return (value < 0.0 ? 1 : 0);
  }

private:
  /// Minimal length of each edge of all elements in the phase field transition;
  double hInitInterface;

  double hInterface;

  /// If non zero, do not use estimator and marking strategy, but instead the interface
  /// refinement algorithm on the solution vector.
  int refInterface;

  double rhoInitial;

  int serial;
};
