/** \file Refinement.h */

#ifndef REFINEMENT_PHASEFIELD_H
#define REFINEMENT_PHASEFIELD_H

#include "AMDiS.h"

using namespace std;
using namespace AMDiS;

/* define instationary problem */
class Refinement
{
public:
  Refinement(DOFVector<double> *phasefield, AbstractFunction<double, WorldVector<double> > *phasefieldFct, RefinementManager *refinementManager, CoarseningManager *coarseningManager);
  ~Refinement();
  
  void refine(double h);
  
  void calcMeshSizes(double &minH, double &maxH);
  void markElements(Flag &markFlag, double hInterface);
  bool refineMesh(Flag markFlag);

protected:
  inline double norm(const WorldVector<double> &x) 
  { 
    return sqrt(x * x); 
  }

protected:
  AbstractFunction<double, WorldVector<double> > *phasefieldFct;
  DOFVector<double> *phasefield;
  const FiniteElemSpace *feSpace;
  Mesh *mesh;
  RefinementManager *refinementManager;
  CoarseningManager *coarseningManager;
  int info_;
};
#endif // REFINEMENT_PHASEFIELD_H
